@@ -132,7 +132,7 @@ function task_install_source_deb_systemd {
 # Debian packaging during the 'install' phases.
 function task_install {
     install -d -m 0755 "${DESTDIR}${app_prefix}"
-    install -d -m 0755 "${DESTDIR}${app_data}"
+    install -d -m 0770 "${DESTDIR}${app_data}"
     install -m 0644 puppetdb.jar "${DESTDIR}${app_prefix}"
     install -m 0755 ext/ezbake-functions.sh "${DESTDIR}${app_prefix}"
     install -m 0644 ext/ezbake.manifest "${DESTDIR}${app_prefix}"
@@ -158,11 +158,14 @@ function task_install {
     ln -s "../apps/${real_name}/bin/${real_name}" "${DESTDIR}${symbindir}/${real_name}"
     install -d -m 0755 "${DESTDIR}${uxbindir}"
     ln -s "../server/apps/${real_name}/bin/${real_name}" "${DESTDIR}${uxbindir}/${real_name}"
+    install -m 0755 ext/cli/reload "${DESTDIR}${app_prefix}/cli/apps/reload"
     install -m 0755 ext/cli/foreground "${DESTDIR}${app_prefix}/cli/apps/foreground"
+    install -m 0755 ext/cli/start "${DESTDIR}${app_prefix}/cli/apps/start"
     install -m 0755 ext/cli/ssl-setup "${DESTDIR}${app_prefix}/cli/apps/ssl-setup"
     install -m 0755 ext/cli/export "${DESTDIR}${app_prefix}/cli/apps/export"
     install -m 0755 ext/cli/config-migration "${DESTDIR}${app_prefix}/cli/apps/config-migration"
     install -m 0755 ext/cli/foreground "${DESTDIR}${app_prefix}/cli/apps/foreground"
+    install -m 0755 ext/cli/stop "${DESTDIR}${app_prefix}/cli/apps/stop"
     install -m 0755 ext/cli/anonymize "${DESTDIR}${app_prefix}/cli/apps/anonymize"
     install -m 0755 ext/cli/import "${DESTDIR}${app_prefix}/cli/apps/import"
     install -d -m 0755 "${DESTDIR}${rundir}"
@@ -310,6 +313,8 @@ function task_postinst_permissions {
     chmod 770 $app_data
     chown puppetdb:puppetdb $projconfdir
     chmod 750 $projconfdir
+    chown puppetdb:puppetdb $rundir
+    chmod 0755 $rundir
 }
 
 # Install logrotate (usually el7, fedora 16 and above)
