@@ -6,16 +6,19 @@
   (:require [puppetlabs.puppetdb.cheshire :as json]
             [clojure.walk :refer [keywordize-keys stringify-keys]]
             [clojure.core.match :as cm]
-            [puppetlabs.puppetdb.query-eng :refer [produce-streaming-body]]
+            [puppetlabs.puppetdb.query-eng :as qeng]
             [clojure.set :as set]
+            [puppetlabs.i18n.core :as i18n]
             [puppetlabs.kitchensink.core :as kitchensink]
-            [net.cgrand.moustache :refer [app]]
             [schema.core :as s]
             [puppetlabs.puppetdb.http :as http]
             [puppetlabs.puppetdb.schema :as pls]
-            [puppetlabs.puppetdb.query.paging :refer [parse-limit' parse-offset' parse-order-by']]
+            [puppetlabs.puppetdb.query.paging :refer [parse-limit
+                                                      parse-offset
+                                                      parse-order-by
+                                                      parse-order-by-json]]
+            [puppetlabs.puppetdb.pql :as pql]
             [puppetlabs.puppetdb.time :refer [to-timestamp]]
-            [clojure.tools.logging :as log]
             [puppetlabs.puppetdb.utils :refer [update-when]]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -27,7 +30,9 @@
   parameters, for POST requests this should be contained in the body
   of the request"
   {(s/optional-key :query) (s/maybe [s/Any])
+   (s/optional-key :ast_only) (s/maybe s/Bool)
    (s/optional-key :include_total) (s/maybe s/Bool)
+   (s/optional-key :pretty) (s/maybe s/Bool)
    (s/optional-key :order_by) (s/maybe [[(s/one s/Keyword "field")
                                          (s/one (s/enum :ascending :descending) "order")]])
    (s/optional-key :distinct_resources) (s/maybe s/Bool)
@@ -46,9 +51,6 @@
   {(s/optional-key :optional) [s/Str]
    (s/optional-key :required) [s/Str]})
 
-(def experimental-entities
-  #{:event-counts :aggregate-event-counts})
-
 (defn- are-queries-different?
   [req1 req2]
   (not= (:puppetdb-query req1)
@@ -75,25 +77,35 @@
           (tree-seq vector? rest criteria))))
 
 (defn add-criteria
-  "Add a criteria to the given query, taking top-level extract queries into
-  account."
+  "Add a criteria to the given query, taking top-level 'extract' and 'from'
+  forms into account."
   [crit query]
-  (cm/match [query]
-    [["extract" columns nil]]
-    ["extract" columns crit]
+  (if-not crit
+    query
+    (cm/match [query]
+      [["extract" columns nil]]
+      ["extract" columns crit]
 
-    [["extract" columns]]
-    ["extract" columns crit]
+      [["extract" columns]]
+      ["extract" columns crit]
 
-    [["extract" columns subquery]]
-    ["extract" columns ["and" subquery crit]]
+      [["extract" columns subquery]]
+      (if (= "group_by" (first subquery))
+        ["extract" columns crit subquery]
+        ["extract" columns ["and" subquery crit]])
 
-    [["extract" columns subquery clauses]]
-    ["extract" columns ["and" subquery crit] clauses]
+      [["extract" columns subquery clauses]]
+      ["extract" columns (add-criteria crit subquery) clauses]
 
-    :else (if query
-            ["and" query crit]
-            crit)))
+      [["from" entity]]
+      ["from" entity crit]
+
+      [["from" entity subquery]]
+      ["from" entity (add-criteria crit subquery)]
+
+      :else (if query
+              ["and" query crit]
+              crit))))
 
 (defn restrict-query
   "Given a criteria that will restrict a query, modify the supplied
@@ -120,65 +132,87 @@
 (defn restrict-query-to-node
   "Restrict the query parameter of the supplied request so that it
    only returns results for the supplied node"
-  [node req]
-  {:pre  [(string? node)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :node]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "certname" node] req))
+  (restrict-query ["=" "certname" (get-in req [:route-params :node])] req))
 
 (defn restrict-query-to-report
   "Restrict the query parameter of the supplied request so that it
    only returns results for the supplied active node"
-  [hash req]
-  {:pre  [(string? hash)]
+  [req]
+  {:pre  [(get-in req [:route-params :hash])]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "report" hash]
+  (restrict-query ["=" "report" (get-in req [:route-params :hash])]
                   req))
 
 (defn restrict-query-to-environment
   "Restrict the query parameter of the supplied request so that it
    only returns results for the supplied environment"
-  [environment req]
-  {:pre  [(string? environment)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :environment]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "environment" environment]
+  (restrict-query ["=" "environment" (get-in req [:route-params :environment])]
+                  req))
+
+(defn restrict-query-to-producer
+  "Restrict the query parameter of the supplied request so that it
+   only returns results for the supplied producer"
+  [req]
+  {:pre  [(string? (get-in req [:route-params :producer]))]
+   :post [(are-queries-different? req %)]}
+  (restrict-query ["=" "producer" (get-in req [:route-params :producer])]
                   req))
 
 (defn restrict-fact-query-to-name
   "Restrict the query parameter of the supplied request so that it
    only returns facts with the given name"
-  [fact req]
-  {:pre  [(string? fact)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :fact]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "name" fact]
+  (restrict-query ["=" "name" (get-in req [:route-params :fact])]
                   req))
 
 (defn restrict-fact-query-to-value
   "Restrict the query parameter of the supplied request so that it
   only returns facts with the given name"
-  [value req]
-  {:pre  [(string? value)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :value]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "value" value]
+  (restrict-query ["=" "value" (get-in req [:route-params :value])]
                   req))
 
 (defn restrict-resource-query-to-type
   "Restrict the query parameter of the supplied request so that it
   only returns resources with the given type"
-  [type req]
-  {:pre  [(string? type)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :type]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "type" type]
+  (restrict-query ["=" "type" (get-in req [:route-params :type])]
                   req))
 
 (defn restrict-resource-query-to-title
   "Restrict the query parameter of the supplied request so that it
    only returns resources with the given title"
-  [title req]
-  {:pre  [(string? title)]
+  [req]
+  {:pre  [(string? (get-in req [:route-params :title]))]
    :post [(are-queries-different? req %)]}
-  (restrict-query ["=" "title" title]
+  (restrict-query ["=" "title" (get-in req [:route-params :title])]
                   req))
 
+(defn wrap-with-from
+  "Wrap a query in a from, using the entity and any provided query"
+  [entity query]
+  (if query
+    ["from" entity query]
+    ["from" entity]))
+
+(pls/defn-validated restrict-query-to-entity
+  "Restrict the query to a particular entity, by wrapping the query in a from."
+  [entity :- String]
+  (fn [req]
+    (update-in req [:puppetdb-query :query] #(wrap-with-from entity %))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Conversion/validation of query parameters
 
@@ -217,45 +251,68 @@
   (-> (or full-query {})
       (validate-query-params param-spec)
       keywordize-keys
-      (update-when [:order_by] parse-order-by')
-      (update-when [:limit] parse-limit')
-      (update-when [:offset] parse-offset')
+      (update-when [:ast_only] coerce-to-boolean)
+      (update-when [:order_by] parse-order-by)
+      (update-when [:limit] parse-limit)
+      (update-when [:offset] parse-offset)
       (update-when [:include_total] coerce-to-boolean)
+      (update-when [:pretty] coerce-to-boolean)
       (update-when [:distinct_resources] coerce-to-boolean)))
 
 (defn get-req->query
   "Converts parameters of a GET request to a pdb query map"
-  [{:keys [params] :as req}]
+  [{:keys [params] :as req}
+   parse-fn]
   (-> params
-      (update-when ["query"] json/parse-strict-string true)
-      (update-when ["order_by"] json/parse-strict-string true)
+      (update-when ["query"] parse-fn)
+      (update-when ["order_by"] parse-order-by-json)
       (update-when ["counts_filter"] json/parse-strict-string true)
+      (update-when ["pretty"] coerce-to-boolean)
       keywordize-keys))
 
 (defn post-req->query
   "Takes a POST body and parses the JSON to create a pdb query map"
-  [req]
-  (with-open [reader (-> req :body clojure.java.io/reader)]
-    (json/parse-stream reader true)))
+  [req parse-fn]
+  (-> (with-open [reader (-> req :body clojure.java.io/reader)]
+        (json/parse-stream reader true))
+      (update :query (fn [query]
+                       (if (vector? query)
+                         query
+                         (parse-fn query))))))
 
 (pls/defn-validated create-query-map :- puppetdb-query-schema
   "Takes a ring request map and extracts the puppetdb query from the
   request. GET and POST are accepted, all other requests throw an
   exception"
-  [req param-spec]
+  [req param-spec parse-fn]
   (convert-query-params
    (case (:request-method req)
-     :get (get-req->query req)
-     :post (post-req->query req)
+     :get (get-req->query req parse-fn)
+     :post (post-req->query req parse-fn)
      (throw (IllegalArgumentException. "PuppetDB queries must be made via GET/POST")))
-    param-spec))
+   param-spec))
 
 (defn extract-query
   "Query handler that converts the incoming request (GET or POST)
   parameters/body to a pdb query map"
+  ([handler param-spec]
+   (extract-query handler param-spec pql/parse-json-query))
+  ([handler param-spec parse-fn]
+   (fn [{:keys [puppetdb-query] :as req}]
+     (handler
+      (if puppetdb-query
+        req
+        (let [param-spec (update param-spec :optional conj "pretty")
+              query-map (create-query-map req param-spec parse-fn)
+              pretty-print (:pretty query-map
+                                    (get-in req [:globals :pretty-print]))]
+          (-> req
+              (assoc :puppetdb-query query-map)
+              (assoc-in [:globals :pretty-print] pretty-print))))))))
+
+(defn extract-query-pql
   [handler param-spec]
-  (fn [{:keys [request-method body params puppetdb-query] :as req}]
-    (handler (assoc req :puppetdb-query (create-query-map req param-spec)))))
+  (extract-query handler param-spec pql/parse-json-or-pql-to-ast))
 
 (defn validate-distinct-options!
   "Validate the HTTP query params related to a `distinct_resources` query.  Return a
@@ -290,30 +347,22 @@
        (IllegalArgumentException.
          "'distinct_resources' query parameter requires accompanying parameters 'distinct_start_time' and 'distinct_end_time'")))))
 
-(defn warn-experimental
-  [entity warn-experimental]
-  (when (and warn-experimental
-             (contains? experimental-entities entity))
-    (log/warn (format
-                "The %s endpoint is experimental and may be altered or removed in the future."
-                (name entity)))))
+(defn narrow-globals
+  "Reduces the number of globals to limit their reach in the codebase"
+  [globals]
+  (select-keys globals [:scf-read-db :warn-experimental :url-prefix :pretty-print]))
 
-(defn query-route
-  "Returns a route for querying PuppetDB that supports the standard
-   paging parameters. Also accepts GETs and POSTs. Composes
-   `optional-handlers` with the middleware function that executes the
-   query."
-  [entity version param-spec & optional-handlers]
-  (let [handlers (or optional-handlers [identity])]
-    (app
-      (extract-query param-spec)
-      (apply comp
-             (fn [{:keys [params globals puppetdb-query]}]
-               (warn-experimental entity (:warn-experimental globals))
-               (produce-streaming-body
-                 entity
-                 version
-                 (validate-distinct-options! (merge (keywordize-keys params) puppetdb-query))
-                 (:scf-read-db globals)
-                 (:url-prefix globals)))
-             optional-handlers))))
+(defn valid-query?
+  [version query-map]
+  (let [{:keys [remaining-query entity query-options]} (qeng/user-query->engine-query version query-map)]
+    (when (qeng/query->sql remaining-query entity version query-options)
+      true)))
+
+(defn query-handler
+  [version]
+  (fn [{:keys [params globals puppetdb-query]}]
+    (if (and (:ast_only puppetdb-query) (valid-query? version puppetdb-query))
+      (http/json-response (:query puppetdb-query))
+      (qeng/produce-streaming-body version
+                              (validate-distinct-options! (merge (keywordize-keys params) puppetdb-query))
+                              (narrow-globals globals)))))
